--- channels/chan_sip.c.orig	2010-03-12 10:02:07.000000000 +0100
+++ channels/chan_sip.c	2010-03-12 10:40:10.000000000 +0100
@@ -1603,6 +1603,7 @@ static enum ast_rtp_get_result sip_get_r
 static enum ast_rtp_get_result sip_get_vrtp_peer(struct ast_channel *chan, struct ast_rtp **rtp);
 static int sip_get_codec(struct ast_channel *chan);
 static struct ast_frame *sip_rtp_read(struct ast_channel *ast, struct sip_pvt *p, int *faxdetect);
+static int sip_rtp_prod(const void *data);
 
 /*------ T38 Support --------- */
 static int sip_handle_t38_reinvite(struct ast_channel *chan, struct sip_pvt *pvt, int reinvite); /*!< T38 negotiation helper function */
@@ -3873,6 +3874,9 @@ static int sip_answer(struct ast_channel
 
 		res = transmit_response_with_sdp(p, "200 OK", &p->initreq, XMIT_CRITICAL);
 		ast_set_flag(&p->flags[1], SIP_PAGE2_DIALOG_ESTABLISHED);
+		/* When both bridged channels send a=direction:passive, we're stuck.
+		 * Prod the channel after a while. Yes, this is an ugly hack :) */
+		ast_sched_add(sched, 1000, sip_rtp_prod, ast);
 	}
 	ast_mutex_unlock(&p->lock);
 	return res;
@@ -4617,6 +4621,38 @@ static struct ast_frame *sip_read(struct
 	return fr;
 }
 
+/*! \brief Poke media session, see sip_answer */
+static int sip_rtp_prod(const void *data)
+{
+	struct ast_channel *chan = (struct ast_channel*)data;
+        struct ast_frame a = { AST_FRAME_VOICE, 0, };
+        char garbage[160];
+
+	/* Fill frame with enough garbage to get at least one UDP packet out. */
+	a.subclass = chan->rawwriteformat;
+	a.data = garbage; /*+ AST_FRIENDLY_OFFSET;*/
+	a.datalen = 160;
+	a.samples = 160;
+	a.len = 20; /* ms */
+
+	a.mallocd = 0;
+	a.src = "sip_rtp_prod";
+
+	/* I don't know what these are supposed to be set to. */
+	a.flags = 1;
+	a.ts = 12345;
+	a.seqno = 12345;
+
+        if (option_debug)
+		ast_log(LOG_DEBUG, "Prodding channel '%s'\n", chan->name);
+
+        ast_channel_lock(chan);
+        if (sip_write(chan, &a))
+        	ast_log(LOG_WARNING, "Prodding channel '%s' failed\n", chan->name);
+        ast_channel_unlock(chan);
+        return 0;
+}
+
 
 /*! \brief Generate 32 byte random string for callid's etc */
 static char *generate_random_string(char *buf, size_t size)
